/*
  labcomm_bytearray_reader.c -- a reader for data in a byte array

  Copyright 2014 Sen Gestegård Robertz <sven.robertz@cs.lth.se>

  This file is part of LabComm.

  LabComm is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  LabComm is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "labcomm_private.h"
#include "labcomm_bytearray_reader.h"


struct labcomm_bytearray_reader {
  struct labcomm_reader reader;
  struct labcomm_reader_action_context action_context;
  unsigned char *bytearray;
  int bytearray_size;
};

static int bytearray_alloc(struct labcomm_reader *r,
		    struct labcomm_reader_action_context *action_context, 
		    char *version)
{
  int result = 0;
  
  r->data = ((struct labcomm_bytearray_reader *)action_context->context)->bytearray;
  r->count =((struct labcomm_bytearray_reader *)action_context->context)->bytearray_size;
  r->pos = 0;
  return result;
}

static int bytearray_free(struct labcomm_reader *r, 
		   struct labcomm_reader_action_context *action_context)
{
  struct labcomm_bytearray_reader *bytearray_reader = action_context->context;
  struct labcomm_memory *memory = r->memory;

  //HERE BE DRAGONS labcomm_memory_free(memory, 0, r->data);
  r->data = 0;
  r->data_size = 0;
  r->count = 0;
  r->pos = 0;

  labcomm_memory_free(memory, 0, bytearray_reader);
  return 0;
}

static int bytearray_fill(struct labcomm_reader *r, 
		   struct labcomm_reader_action_context *action_context)
{
  int result = 0;

  if (r->pos < r->count) {
    //printf("BAreader.fill: pos %d count: %d\n", r->pos, r->count);
    result = r->count - r->pos;
  } else {
    r->pos = 0;
    r->error = -EPIPE;
    result = -EPIPE;
  }
  return result;
}

static const struct labcomm_reader_action action = {
  .alloc = bytearray_alloc,
  .free = bytearray_free,
  .start = NULL,
  .fill = bytearray_fill,
  .end = NULL,
  .ioctl = NULL
};

struct labcomm_reader *labcomm_bytearray_reader_new(struct labcomm_memory *memory,
					     unsigned char *data, int size)
{
  struct labcomm_bytearray_reader *result;

  result = labcomm_memory_alloc(memory, 0, sizeof(*result));
  if (result == NULL) {
    return NULL;
  } else {
    result->action_context.next = NULL;
    result->action_context.action = &action;
    result->action_context.context = result;
    result->reader.action_context = &result->action_context;
    result->reader.memory = memory;
    result->bytearray = data;
    result->bytearray_size = size;
    struct labcomm_reader *   res = &result->reader;
   // printf("reader: %p, size: %d\n", res, res->data_size);
    return res;
  }
}
